//! Stores a DEFINE PARAM config definition
use crate::key::error::KeyCategory;
use crate::key::key_req::KeyRequirements;
use derive::Key;
use serde::{Deserialize, Serialize};

#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Serialize, Deserialize, Key)]
pub struct Pa<'a> {
	__: u8,
	_a: u8,
	pub ns: &'a str,
	_b: u8,
	pub db: &'a str,
	_c: u8,
	_d: u8,
	_e: u8,
	pub pa: &'a str,
}

pub fn new<'a>(ns: &'a str, db: &'a str, pa: &'a str) -> Pa<'a> {
	Pa::new(ns, db, pa)
}

pub fn prefix(ns: &str, db: &str) -> Vec<u8> {
	let mut k = super::all::new(ns, db).encode().unwrap();
	k.extend_from_slice(&[b'!', b'p', b'a', 0x00]);
	k
}

pub fn suffix(ns: &str, db: &str) -> Vec<u8> {
	let mut k = super::all::new(ns, db).encode().unwrap();
	k.extend_from_slice(&[b'!', b'p', b'a', 0xff]);
	k
}

impl KeyRequirements for Pa<'_> {
	fn key_category(&self) -> KeyCategory {
		KeyCategory::DatabaseParameter
	}
}

impl<'a> Pa<'a> {
	pub fn new(ns: &'a str, db: &'a str, pa: &'a str) -> Self {
		Self {
			__: b'/',
			_a: b'*',
			ns,
			_b: b'*',
			db,
			_c: b'!',
			_d: b'p',
			_e: b'a',
			pa,
		}
	}
}

#[cfg(test)]
mod tests {
	#[test]
	fn key() {
		use super::*;
		#[rustfmt::skip]
		let val = Pa::new(
			"testns",
			"testdb",
			"testpa",
		);
		let enc = Pa::encode(&val).unwrap();
		assert_eq!(enc, b"/*testns\0*testdb\0!patestpa\0");

		let dec = Pa::decode(&enc).unwrap();
		assert_eq!(val, dec);
	}
}
