//! The module defining the key value store.
//! Everything related the transaction for the key value store is defined in the `tx.rs` file.
//! This module enables the following operations on the key value store:
//! - get
//! - set
//! - delete
//! - put
//! These operations can be processed by the following storage engines:
//! - `fdb`: [FoundationDB](https://github.com/apple/foundationdb/) a distributed database designed to handle large volumes of structured data across clusters of commodity servers
//! - `indxdb`: WASM based database to store data in the browser
//! - `rocksdb`: [RocksDB](https://github.com/facebook/rocksdb) an embeddable persistent key-value store for fast storage
//! - `speedb`: [SpeedyDB](https://github.com/speedb-io/speedb) fork of rocksDB making it faster (Redis is using speedb but this is not acid transactions)
//! - `tikv`: [TiKV](https://github.com/tikv/tikv) a distributed, and transactional key-value database
//! - `mem`: in-memory database
mod cache;
mod ds;
mod fdb;
mod indxdb;
mod kv;
// pub(crate) for tests
pub(crate) mod mem;
mod rocksdb;
mod speedb;
mod surrealkv;
mod tikv;
mod tx;

mod clock;
pub(crate) mod lq_structs;
#[cfg(test)]
#[cfg(any(
	feature = "kv-mem",
	feature = "kv-rocksdb",
	feature = "kv-speedb",
	feature = "kv-indxdb",
	feature = "kv-tikv",
	feature = "kv-fdb",
	feature = "kv-surrealkv"
))]
mod tests;

pub use self::ds::*;
pub use self::kv::*;
pub use self::tx::*;
